<?
/* Name: summary.php
 * Project: Remote Sensor based on AVR USB driver
 * Author: Christian Starkjohann
 * Creation Date: 2005-04-13
 * Tabsize: 4
 * Copyright: (c) 2005 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt) or proprietary (CommercialLicense.txt)
 * This Revision: $Id: summary.php 314 2007-03-29 17:17:09Z cs $
 */

include_once("functions.php");

/*
General Description:
This script builds a chain of data files with averages of the temperature and
humidity values in the sensor daemon's log file. The averaged files are used
for quick creation of long term diagrams on the fly. See the script
'diagram.php' for details. The names of the data files are defined in the
file 'functions.php'.
*/

$lockdir = "$workdir/summary.lock";

function lockWithDir($path)
{
    while(!@mkdir($path, 0770)){    /* mkdir should be atomic and fail if dir exists */
        sleep(1);
        $attr = @stat($path);
        if(!$attr)          /* directory has probably been removed */
            continue;
        if($attr[9] + 45 < time()){
            /* locked section may take 45 seconds at maximum */
            rmdir($path);   /* remove stale lock dir */
        }
    }
}

function unlockWithDir($path)
{
    rmdir($path);
}

function    printRecord($fp, $time, $counts, $temps, $humids)
{
    if(count($counts) > 0){ /* if there are any elements in the arrays */
        $s = sprintf("%d", $time);
        for($i=0;$i<16;$i++){
            if($counts[$i] > 0){
                $s .= sprintf(";%.1f;%.0f", $temps[$i] / $counts[$i], $humids[$i] / $counts[$i]);
            }else{
                $s .= ";*;*";
            }
        }
        fwrite($fp, "$s\n");
    }
}

    lockWithDir($lockdir);
    $files = array($avg30m, $avg2h, $avg1d, $avg7d);
    $intervals = array(30*60, 120*60, 86400, 7*86400);
    $refFile = $csvfile;
    for($i=0;$i<count($files);$i++){
        $file = $files[$i];
        $avgInterval = $intervals[$i];
        if(file_exists($file)){
            $avgStart = lastTimeInFile($file) + $avgInterval;
        }else{
            $avgStart = 0;
        }
        $ref = openAtTime($refFile, $avgStart);
        if(!$ref){
            echo "File $refFile does not exist!\n";
            break;
        }
        $fp = fopen($file, "a");
        $humid = $temp = $cnt = array();
        while(($a = fgetcsv($ref, 1024, ";")) !== FALSE){
            if($refFile == $csvfile){   /* different file format here... */
                $t = parseTimestamp($a[0]);
                if(count($a) < 5 || $t == 0){   /* parse error */
                    echo "Parse error in $refFile\n";
                    continue;
                }
                if($t - $avgStart >= $avgInterval){
                    printRecord($fp, $avgStart, $cnt, $temp, $humid);
                    $avgStart = ((int)($t / $avgInterval)) * $avgInterval;
                    $humid = $temp = $cnt = array();
                }
                $id = (int)$a[1];
                $cnt[$id]++;
                $temp[$id] += $a[3];
                $humid[$id] += $a[4];
            }else{
                if(count($a) < 33){ /* parse error */
                    echo "Parse error in $refFile\n";
                    continue;
                }
                if($a[0] - $avgStart >= $avgInterval){
                    printRecord($fp, $avgStart, $cnt, $temp, $humid);
                    $avgStart = ((int)($a[0] / $avgInterval)) * $avgInterval;
                    $humid = $temp = $cnt = array();
                }
                for($j=0;$j<16;$j++){
                    if($a[2 * $j + 1] != '*' && $a[2 * $j + 2] != '*'){
                        $cnt[$j]++;
                        $temp[$j] += $a[2 * $j + 1];
                        $humid[$j] += $a[2 * $j + 2];
                    }
                }
            }
        }
        fclose($ref);
        fclose($fp);
        $refFile = $file;   /* build next average from this average */
    }
    unlockWithDir($lockdir);
?>
