/* Name: txasm.S
 * Project: Remote Sensor; transmitter
 * Author: Christian Starkjohann
 * Creation Date: 2005-03-20
 * Tabsize: 4
 * Copyright: (c) 2005 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt) or proprietary (CommercialLicense.txt)
 * This Revision: $Id: txasm.S 312 2007-03-29 17:15:25Z cs $
 */

/* configs for io.h */
#define __SFR_OFFSET 0
#define _VECTOR(N) __vector_ ## N   /* io.h does not define this for asm */

#include <avr/io.h> /* for CPU I/O register definitions and vectors */
#include "hardware.h"

#define nop2    rjmp .+0

.text

; extern unsigned crc16(unsigned char *data, unsigned char len);
; CRC with polynomial from USB spec, returns inverted CRC.
; data: r24/25
; len: r22
; temp variables:
;   r18: data byte
;   r19: bit counter
;   r20/21: polynomial
;   r23: scratch
;   r24/25: crc-sum
;   r26/27=X: ptr
.global crc16
crc16:
    mov     XL, r24
    mov     XH, r25
    ldi     r24, 0xff
    ldi     r25, 0xff
    ldi     r20, lo8(0xa001)
    ldi     r21, hi8(0xa001)
crcByteLoop:
    subi    r22, 1
    brcs    crcReady
    ld      r18, x+
    ldi     r19, 8
crcBitLoop:
    mov     r23, r18
    eor     r23, r24
    lsr     r25
    ror     r24
    lsr     r18
    sbrs    r23, 0
    rjmp    crcNoXor
    eor     r24, r20
    eor     r25, r21
crcNoXor:
    dec     r19
    brne    crcBitLoop
    rjmp    crcByteLoop
crcReady:
    com     r24
    com     r25
    ret

; ----------------------------------------------------------------------------

#define shift   r18
#define data    r19
#define cnt     r20
#define bcnt    r21
#define xPll    r24
#define xCnt    r25

; A bit rate of 6.5536 kbit/s (= 5 CPU cycles per bit) would be possible with
; the unrolled loop construct commented out below.
; Since our receiver has a bandwidth limit of about 2 kHz, we limit ourselves
; to half the possible bit rate (3.2768 kbit/s = 10 CPU cycles per bit).
; void uartTxBlock(char *block, unsigned char len);
;                      R25:R24                R22
.global uartTxBlock
uartTxBlock:
    mov     XL, R24
    mov     XH, R25
    mov     cnt, r22            ;load byte count
    in      data, UART_PORT
    ori     data, 1<<UART_BIT   ; stop bit set
    out     UART_PORT, data     ; <- stop bit
    ldi     bcnt, 8             ; prefix >8 stop bits of 10 cycles each = 80 cycles
.delayLoop:
    ldi     xPll, 0xff          ;1 **PWR
    ldi     xCnt, CNT1MODE      ;1 **PWR
    sbis    ACSR, ACO           ;1 **PWR bit is set if we need PWM
    clr     xPll                ;1 **PWR
    and     xCnt, xPll          ;1 **PWR
    andi    xPll, PLLMODE       ;1 **PWR
    out     TCCR1A, xCnt        ;1 **PWR
    out     PLLCSR, xPll        ;1 **PWR
    dec     bcnt                ;1
    brne    .delayLoop          ;2
.byteLoop:
    andi    data, ~(1<<UART_BIT);1 start bit
    out     UART_PORT, data     ;1 <- start bit
    ld      shift, X+           ;2
    ldi     bcnt, 4             ;1
.bitloop:
    nop                         ;1
    ldi     xPll, 0xff          ;1 **PWR
    ldi     xCnt, CNT1MODE      ;1 **PWR
    bst     shift, 0            ;1
    bld     data, UART_BIT      ;1
    ror     shift               ;1
    out     UART_PORT, data     ;1 <- bit
    sbis    ACSR, ACO           ;1 **PWR bit is set if we need PWM
    clr     xPll                ;1 **PWR
    and     xCnt, xPll          ;1 **PWR
    andi    xPll, PLLMODE       ;1 **PWR
    out     TCCR1A, xCnt        ;1 **PWR
    out     PLLCSR, xPll        ;1 **PWR
    bst     shift, 0            ;1
    bld     data, UART_BIT      ;1
    ror     shift               ;1
    out     UART_PORT, data     ;1 <- bit
    dec     bcnt                ;1
    brne    .bitloop            ;2
                                ;-1 for branch not taken
    nop2                        ;2
    nop2                        ;2
    nop2                        ;2
    ori     data, 1 << UART_BIT ;1 stop bit
    out     UART_PORT, data     ;1 <- stop bit
    nop                         ;1
    nop2                        ;2
    nop2                        ;2
    dec     cnt                 ;1
    brne    .byteLoop           ;2
    clr     xPll                ;1 **PWR turn off power
    out     TCCR1A, xPll        ;1 **PWR
    out     PLLCSR, xPll        ;1 **PWR
    andi    data, ~(1<<UART_BIT);1 turn off transmitter
    out     UART_PORT, data     ;1
    ret

; ----------------------------------------------------------------------------

.global SIG_OVERFLOW0
    .type   SIG_OVERFLOW0, @function
SIG_OVERFLOW0:
    push    r16
    ldi     r16, 1
    sts     hadTimerInterrupt, r16
    pop     r16
    reti

; ----------------------------------------------------------------------------

.global SIG_ADC
    .type SIG_ADC, @function
SIG_ADC:
    reti

; ----------------------------------------------------------------------------

#if 0
; This is a double-speed transmitter routine. Unfortunately, our tx/rx modules
; are too slow for this.
uartTxBlock:
    mov     XL, R24
    mov     XH, R25         ; X = ptr to block
    in      data, UART_PORT
    ori     data, 0x01      ; stop bit set
    out     UART_PORT, data ; stop bit
    ldi     cnt, 4          ; prefix >8 stop bits of 5 cycles each = 40 cycles
.delayLoop:
    ldi     xPll, 0xff      ;1 **PWR
    ldi     xCnt, CNT1MODE  ;1 **PWR
    sbis    ACSR, ACO       ;1 **PWR bit is set if we need PWM
    clr     xPll            ;1 **PWR
    and     xCnt, xPll      ;1 **PWR
    andi    xPll, PLLMODE   ;1 **PWR
    out     TCCR1A, xCnt    ;1 **PWR
    out     PLLCSR, xPll    ;1 **PWR
    dec     cnt             ;1
    brne    .delayLoop      ;2
    mov     cnt, r22        ; load byte count
.byteloop:
    andi    data, 0xfe      ;1 start bit
    out     UART_PORT, data ;1  -> start bit

    ld      shift, X+       ;2
    bst     shift, 0        ;1
    bld     data, UART_BIT  ;1
    out     UART_PORT, data ;1  -> bit 0

    ldi     xPll, 0xff      ;1 **PWR
    ldi     xCnt, CNT1MODE  ;1 **PWR
    bst     shift, 1        ;1
    bld     data, UART_BIT  ;1
    out     UART_PORT, data ;1  -> bit 1

    bst     shift, 2        ;1
    bld     data, UART_BIT  ;1
    sbis    ACSR, ACO       ;1 **PWR bit is set if we need PWM
    clr     xPll            ;1 **PWR
    out     UART_PORT, data ;1  -> bit 2

    and     xCnt, xPll      ;1 **PWR
    andi    xPll, PLLMODE   ;1 **PWR
    bst     shift, 3        ;1
    bld     data, UART_BIT  ;1
    out     UART_PORT, data ;1  -> bit 3

    out     TCCR1A, xCnt    ;1 **PWR
    out     PLLCSR, xPll    ;1 **PWR
    bst     shift, 4        ;1
    bld     data, UART_BIT  ;1
    out     UART_PORT, data ;1  -> bit 4

    ldi     xPll, 0xff      ;1 **PWR
    ldi     xCnt, CNT1MODE  ;1 **PWR
    bst     shift, 5        ;1
    bld     data, UART_BIT  ;1
    out     UART_PORT, data ;1  -> bit 5

    bst     shift, 6        ;1
    bld     data, UART_BIT  ;1
    sbis    ACSR, ACO       ;1 **PWR bit is set if we need PWM
    clr     xPll            ;1 **PWR
    out     UART_PORT, data ;1  -> bit 6

    and     xCnt, xPll      ;1 **PWR
    andi    xPll, PLLMODE   ;1 **PWR
    bst     shift, 7        ;1
    bld     data, UART_BIT  ;1
    out     UART_PORT, data ;1  -> bit 7
    
    out     TCCR1A, xCnt    ;1 **PWR
    out     PLLCSR, xPll    ;1 **PWR
    nop                     ;1
    ori     data, 0x01      ;1 stop bit
    out     UART_PORT, data ;1  -> stop bit

    dec     cnt             ;1
    brne    .byteloop       ;2
    clr     xPll            ;1 **PWR turn off power
    out     TCCR1A, xPll    ;1 **PWR
    out     PLLCSR, xPll    ;1 **PWR
    andi    data, 0xfe      ;1 turn off transmitter
    out     UART_PORT, data ;1
    ret
#endif
